/*
    dsproxy.h - OSS trampoline device driver

    Copyright (C) 2000 Ram Sivrasubramanium <ram@linux-workshop.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef DSPROXY_H
#define DSPROXY_H

#define DSPROXY_TCP_PORT 9138

#define DSPROXY_CAPABILITIES (AFMT_MU_LAW | \
		AFMT_U8 | AFMT_S16_LE | AFMT_S16_BE | AFMT_S16_NE | \
		AFMT_S8 | AFMT_U16_LE | AFMT_U16_BE )

#define DSPROXY_STREAM_DSPDATA 	0
#define DSPROXY_STREAM_IOCTL	1
#define DSPROXY_MIXER_IOCTL	2
#define DSPROXY_READER_DISCONNECT	3

#define DSPROXY_MIXER_STEREO_VOLUME(x) ((x<<8)|(x))
#define DSPROXY_SOUND_MASK (SOUND_MASK_VOLUME | SOUND_MASK_BASS | SOUND_MASK_TREBLE | SOUND_MASK_PCM | SOUND_MASK_OGAIN)


#ifndef __KERNEL__
#define DSPROXY_FIFO_SIZE (4*4096)		/* 131072 keep this arbitrarily large so user mode apps can cope with different sizes of kernel buffers */
#else
#define DSPROXY_FIFO_SIZE (12*4096)			/* this is for overflow..useful *really* is DSPROXY_FRAGFIFO_SIZE */
#define DSPROXY_FRAGFIFO_SIZE (DSPROXY_FIFO_SIZE>>1)	/* so we'll definitely always have the amount we report back */

#include <linux/malloc.h>
#include <linux/mm.h>
#include <linux/soundcard.h>
#include <asm/semaphore.h>

#define DSPROXY_FAULTBUF_SIZ 65536
#define DSPROXY_FRAG_SIZ 1024
#define DSPROXY_HDRSIZ		((sizeof(char))+(sizeof(short)))

#define DSPROXY_MAJOR 121
#define DSPROXY_SUBDEVICE_AUDIO 2
#define DSPROXY_SUBDEVICE_MIXER 3
#define DSPROXY_LOGNAME "dsproxy"

# if defined( DEBUG )
#define DEBUG(x...) x
#else 
#define DEBUG(x...) 
#endif


struct dsproxy_s
{
struct dsproxy_s *next;
uid_t uid;	/* uid of writer */
int clients;
int subdevice;

struct semaphore sem, wr_sem, ioctl_sem, mixer_sem;
struct semaphore forkread, forkwrite; /* bracket r/w ops with these to make forked reads/writes/ioctls sequential! */

int r_inuse;
int w_inuse;
    
/* Audio */
int fragsize;
int numfrags;
int bufsiz;
char frag_explicitly_set;
char send_reset;

int format;
int freq;
int channels, bytes_per_sample;

/* buffers */
char vmbuf[DSPROXY_FIFO_SIZE];
char faultbuf[DSPROXY_FAULTBUF_SIZ+DSPROXY_FRAG_SIZ];	/* this is for programs that don't look at the return value for write() as they should! */
int offset, lowwater;

int fifoget, fifoput, fifocount;
int pcm_outstanding;

/* statistics */
unsigned long long bytes_processed;
unsigned long long hdrskip;

/* sleep */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
wait_queue_head_t inq, outq;
#else 
struct wait_queue *inq, *outq;
#endif 

/* timing */
struct timeval firstwrite, etime;

/* mixer */
int volume;
int bass;
int treble;
int pcm;
int ogain;

int dirty;	/* set when there are dirty bits in the mixer */

/* kernel */
struct inode *inode;
};


extern struct dsproxy_s *emhead;
extern struct semaphore global_sem;


/* dsproxy_audio.c */
int dsproxy_audio_ioctl(struct file *file, struct dsproxy_s *em, unsigned int cmd, unsigned long arg);
int dsproxy_audio_open(struct inode* inode, struct file* filp);
int dsproxy_audio_release(struct dsproxy_s *em, struct file *filp, int subdevice);
int dsproxy_audio_setup(struct dsproxy_s *em);
int dsproxy_audio_write(struct file *file, struct dsproxy_s *em, const char *buf, size_t count, loff_t *ppos);
int dsproxy_audio_read(struct file *file, struct dsproxy_s *em, char *buf, size_t count, loff_t *ppos);
int dsproxy_audio_emit_ioctl(int typ, struct file *file, struct dsproxy_s *em, unsigned int cmd, unsigned int val);
unsigned int dsproxy_audio_poll(struct file * filp, struct poll_table_struct *wait);

int dsproxy_audio_write_bytes(struct dsproxy_s *em, struct file *file, int count2, int offset_orig, int lowwater, char ignore_nonblock);
void dsproxy_audio_flush(struct dsproxy_s *em, struct file *file);

/* dsproxy_mixer.c */
int dsproxy_mixer_ioctl(struct file *file, struct dsproxy_s *emix, unsigned int cmd, unsigned long arg);
int dsproxy_mixer_open(struct inode* inode, struct file* filp);
int dsproxy_mixer_release(struct dsproxy_s *em, struct file *filp, int subdevice);
void dsproxy_mixer_free(void);

#endif	/* __KERNEL__ */
#endif  /* DSPROXY_H  */
